@@ -10,7 +10,7 @@ BEGIN {
 
 BEGIN {
 	$Type::Params::AUTHORITY = 'cpan:TOBYINK';
-	$Type::Params::VERSION   = '1.008005';
+	$Type::Params::VERSION   = '1.010003';
 }
 
 $Type::Params::VERSION =~ tr/_//d;
@@ -35,9 +35,7 @@ sub english_list {
 	goto \&Type::Utils::english_list;
 }
 
-my $QUOTE = ($^V < 5.010 && exists(&B::cstring))
-	? \&B::cstring
-	: \&B::perlstring;   # is buggy on Perl 5.8
+my $QUOTE = \&B::perlstring;
 
 {
 	my $Invocant;
@@ -106,38 +104,149 @@ sub _mkdefault
 	$default;
 }
 
+sub _deal_with_head_and_tail {
+	my $options = shift;
+	$options->{arg_fudge_factor} = 0;
+	$options->{arg_fudge_factor_head} = 0;
+	my @lines;
+	my %env;
+	
+	for my $position (qw/ head tail /) {
+		next unless defined $options->{$position};
+		
+		$options->{$position} = [ (Types::Standard::Any) x (0+ $options->{$position}) ]
+			if !ref $options->{$position};
+			
+		my $count = @{ $options->{$position} };
+		$options->{arg_fudge_factor}      += $count;
+		$options->{arg_fudge_factor_head} += $count if $position eq 'head';
+		
+		push @lines => (
+			$position eq 'head'
+				? "\@head = splice(\@_, 0, $count);"
+				: "\@tail = splice(\@_, -$count);",
+		);
+		
+		for my $i (0 .. $count-1) {
+			my $constraint = $options->{$position}[$i];
+			$constraint = $options->{$position}[$i] = Types::Standard::Any
+				if !ref $constraint && $constraint eq 1;
+			Types::TypeTiny::TypeTiny->assert_valid($constraint);
+			
+			my $is_optional = 0;
+			$is_optional += grep $_->{uniq} == Optional->{uniq}, $constraint->parents;
+			Error::TypeTiny::croak("The $position may not contain optional parameters") if $is_optional;
+			
+			my $varname     = sprintf('$%s[%d]', $position, $i);
+			my $display_var = $position eq 'head'
+				? sprintf('$_[%d]', $i)
+				: sprintf('$_[%d]', $i-$count);
+			
+			if ($constraint->has_coercion and $constraint->coercion->can_be_inlined)
+			{
+				push @lines, sprintf(
+					'%s = do { %s };',
+					$varname,
+					$constraint->coercion->inline_coercion($varname),
+				);
+			}
+			elsif ($constraint->has_coercion)
+			{
+				$env{'@coerce_'.$position}[$i] = $constraint->coercion->compiled_coercion;
+				push @lines, sprintf(
+					'%s = $coerce_%s[%d]->(%s);',
+					$varname,
+					$position,
+					$i,
+					$varname,
+				);
+			}
+			
+			undef $Type::Tiny::ALL_TYPES{ $constraint->{uniq} };
+			$Type::Tiny::ALL_TYPES{ $constraint->{uniq} } = $constraint;
+			
+			if ($constraint == Types::Standard::Any)
+			{
+				# don't really need to check!
+			}
+			elsif ($constraint->can_be_inlined)
+			{
+				push @lines, sprintf(
+					'(%s) or Type::Tiny::_failed_check(%d, %s, %s, varname => %s);',
+					$constraint->inline_check($varname),
+					$constraint->{uniq},
+					$QUOTE->($constraint),
+					$varname,
+					$QUOTE->($display_var),
+				);
+			}
+			else
+			{
+				$env{'@check_'.$position}[$i] = $constraint->compiled_check;
+				push @lines, sprintf(
+					'%s or Type::Tiny::_failed_check(%d, %s, %s, varname => %s);',
+					sprintf(sprintf '$check_%s[%d]->(%s)', $position, $i, $varname),
+					$constraint->{uniq},
+					$QUOTE->($constraint),
+					$varname,
+					$QUOTE->($display_var),
+				);
+			}
+		}
+	}
+	
+	if (@lines) {
+		unshift @lines => sprintf(
+			'"Error::TypeTiny::WrongNumberOfParameters"->throw("Not enough parameters to satisfy required head and tail of parameter list") if @_ < %d;',
+			$options->{arg_fudge_factor},
+		);
+		unshift @lines, 'my (@head, @tail);';
+	}
+	
+	\%env, @lines;
+}
+
 sub compile
 {
 	my (@code, %env);
+	
 	push @code, '#placeholder', '#placeholder';  # @code[0,1]
 	
 	my %options;
 	while (ref($_[0]) eq "HASH" && !$_[0]{slurpy}) {
 		%options = (%options, %{+shift});
 	}
+	
 	my $arg        = -1;
 	my $saw_slurpy = 0;
 	my $min_args   = 0;
 	my $max_args   = 0;
 	my $saw_opt    = 0;
 	
-	my $return_default_list = !!1;
+	my $return_list = '@_';
 	$code[0] = 'my (%tmp, $tmp);';
 	PARAM: for my $param (@_) {
 		if (HashRef->check($param)) {
 			$code[0] = 'my (@R, %tmp, $tmp, $dtmp);';
-			$return_default_list = !!0;
+			$return_list = '@R';
 			last PARAM;
 		}
 		elsif (not Bool->check($param)) {
 			if ($param->has_coercion) {
 				$code[0] = 'my (@R, %tmp, $tmp, $dtmp);';
-				$return_default_list = !!0;
+				$return_list = '@R';
 				last PARAM;
 			}
 		}
 	}
 	
+	my ($extra_env, @extra_lines) = _deal_with_head_and_tail(\%options);
+	if (@extra_lines) {
+		push @code, @extra_lines;
+		%env = (%$extra_env, %env);
+		$return_list = '(@head, @R, @tail)';
+	}
+	
 	my @default_indices;
 	my @default_values;
 		
@@ -214,7 +323,7 @@ sub compile
 			{
 				push @code, sprintf(
 					'return %s if $#_ < %d;',
-					$return_default_list ? '@_' : '@R',
+					$return_list,
 					$arg,
 				);
 				$saw_opt++;
@@ -265,7 +374,7 @@ sub compile
 				$constraint->{uniq},
 				$QUOTE->($constraint),
 				$varname,
-				$is_slurpy ? 'q{$SLURPY}' : sprintf('q{$_[%d]}', $arg),
+				$is_slurpy ? 'q{$SLURPY}' : sprintf('q{$_[%d]}', $arg+$options{arg_fudge_factor_head}),
 			);
 		}
 		else
@@ -279,49 +388,49 @@ sub compile
 				$constraint->{uniq},
 				$QUOTE->($constraint),
 				$varname,
-				$is_slurpy ? 'q{$SLURPY}' : sprintf('q{$_[%d]}', $arg),
+				$is_slurpy ? 'q{$SLURPY}' : sprintf('q{$_[%d]}', $arg+$options{arg_fudge_factor_head}),
 			);
 		}
 		
-		unless ($return_default_list) {
+		unless ($return_list eq '@_') {
 			push @code, sprintf 'push @R, %s;', $varname;
 		}
 	}
-		
+	
+	my $thang = 'scalar(@_)';
+	
 	if ($min_args == $max_args and not $saw_slurpy)
 	{
 		$code[1] = sprintf(
-			'"Error::TypeTiny::WrongNumberOfParameters"->throw(got => scalar(@_), minimum => %d, maximum => %d) if @_ != %d;',
-			$min_args,
-			$max_args,
-			$min_args,
+			'"Error::TypeTiny::WrongNumberOfParameters"->throw(got => %s, minimum => %d, maximum => %d) if @_ != %d;',
+			$thang,
+			$min_args + $options{arg_fudge_factor},
+			$max_args + $options{arg_fudge_factor},
+			$min_args + $options{arg_fudge_factor},
 		);
 	}
 	elsif ($min_args < $max_args and not $saw_slurpy)
 	{
 		$code[1] = sprintf(
-			'"Error::TypeTiny::WrongNumberOfParameters"->throw(got => scalar(@_), minimum => %d, maximum => %d) if @_ < %d || @_ > %d;',
-			$min_args,
-			$max_args,
-			$min_args,
-			$max_args,
+			'"Error::TypeTiny::WrongNumberOfParameters"->throw(got => %s, minimum => %d, maximum => %d) if @_ < %d || @_ > %d;',
+			$thang,
+			$min_args + $options{arg_fudge_factor},
+			$max_args + $options{arg_fudge_factor},
+			$min_args + $options{arg_fudge_factor},
+			$max_args + $options{arg_fudge_factor},
 		);
 	}
 	elsif ($min_args and $saw_slurpy)
 	{
 		$code[1] = sprintf(
-			'"Error::TypeTiny::WrongNumberOfParameters"->throw(got => scalar(@_), minimum => %d) if @_ < %d;',
-			$min_args,
-			$min_args,
+			'"Error::TypeTiny::WrongNumberOfParameters"->throw(got => %s, minimum => %d) if @_ < %d;',
+			$thang,
+			$min_args + $options{arg_fudge_factor},
+			$min_args + $options{arg_fudge_factor},
 		);
 	}
 	
-	if ($return_default_list) {
-		push @code, '@_;';
-	}
-	else {
-		push @code, '@R;';
-	}
+	push @code, $return_list;
 	
 	my $source  = "sub { no warnings; ".join("\n", @code)." };";
 	
@@ -334,8 +443,8 @@ sub compile
 	);
 	
 	return {
-		min_args    => $min_args,
-		max_args    => $saw_slurpy ? undef : $max_args,
+		min_args    => $options{arg_fudge_factor}+($min_args||0),
+		max_args    => $saw_slurpy ? undef : $options{arg_fudge_factor}+($max_args||0),
 		closure     => $closure,
 		source      => $source,
 		environment => \%env,
@@ -348,7 +457,7 @@ sub compile_named
 {
 	my (@code, %env);
 	
-	@code = 'my (%R, %tmp, $tmp);';
+	push @code, 'my (%R, %tmp, $tmp);';
 	push @code, '#placeholder';   # $code[1]
 	
 	my %options;
@@ -357,7 +466,13 @@ sub compile_named
 	}
 	my $arg = -1;
 	my $had_slurpy;
-	
+
+	my ($extra_env, @extra_lines) = _deal_with_head_and_tail(\%options);
+	if (@extra_lines) {
+		push @code, @extra_lines;
+		%env = (%$extra_env, %env);
+	}
+
 	push @code, 'my %in = ((@_==1) && ref($_[0]) eq "HASH") ? %{$_[0]} : (@_ % 2) ? "Error::TypeTiny::WrongNumberOfParameters"->throw(message => "Odd number of elements in hash") : @_;';
 	my @names;
 	
@@ -512,6 +627,13 @@ sub compile_named
 		push @code, '\\%R;';
 	}
 	
+	if ($options{head} || $options{tail}) {
+		$code[-1] = 'my @R = ' . $code[-1];
+		push @code, 'unshift @R, @head;' if $options{head};
+		push @code, 'push @R, @tail;' if $options{tail};
+		push @code, '@R;';
+	}
+	
 	my $source  = "sub { no warnings; ".join("\n", @code)." };";
 	return $source if $options{want_source};
 	
@@ -521,9 +643,15 @@ sub compile_named
 		environment => \%env,
 	);
 	
+	my $max_args = undef;
+	if (!$had_slurpy) {
+		$max_args = 2 * ($arg+1);
+		$max_args += $options{arg_fudge_factor};
+	}
+	
 	return {
-		min_args    => undef,  # always going to be 1 or 0
-		max_args    => undef,  # should be possible to figure out if no slurpy param
+		min_args    => $options{arg_fudge_factor},
+		max_args    => $max_args,
 		closure     => $closure,
 		source      => $source,
 		environment => \%env,
@@ -909,6 +1037,60 @@ options are:
 
 =over
 
+=item C<< head >> B<< Int|ArrayRef[TypeTiny] >>
+
+Parameters to shift off C<< @_ >> before doing the main type check.
+These parameters may also be checked, and cannot be optional or slurpy.
+They may not have defaults.
+
+  my $check = compile(
+    { head => [ Int, Int ] },
+    Str,
+    Str,
+  );
+  
+  # ... is basically the same as...
+  
+  my $check = compile(
+    Int,
+    Int,
+    Str,
+    Str,
+  );
+
+A number may be given if you do not care to check types:
+
+  my $check = compile(
+    { head => 2 },
+    Str,
+    Str,
+  );
+  
+  # ... is basically the same as...
+  
+  my $check = compile(
+    Any,
+    Any,
+    Str,
+    Str,
+  );
+
+This is mostly useless for C<compile>, but can be useful for
+C<compile_named> and C<compile_named_oo>.
+
+=item C<< tail >> B<< Int|ArrayRef[TypeTiny] >>
+
+Similar to C<head>, but pops parameters off the end of C<< @_ >> instead.
+This is actually useful for C<compile> because it allows you to sneak in
+some required parameters I<after> a slurpy or optional parameter.
+
+  my $check = compile(
+    { tail => [ CodeRef ] },
+    slurpy ArrayRef[Str],
+  );
+  
+  my ($strings, $coderef) = $check->("foo", "bar", sub { ... });
+
 =item C<< want_source >> B<< Bool >>
 
 Instead of returning a coderef, return Perl source code string. Handy
@@ -1007,7 +1189,7 @@ I<< not allowed as defaults >>.
 
 Alternatively, you may provide a coderef to generate a default value:
 
- state $check = named(
+ state $check = compile(
    Int,
    Int, { default => sub { 6 * 111 } },
    Int, { default => sub { 9 * 111 } },
@@ -1119,11 +1301,11 @@ parameter:
 
  state $check = compile_named(
    \%general_opts,
-   foo => $type_for_foo, \%opts_for_foo,
-   bar => $type_for_bar, \%opts_for_bar,
-   baz => $type_for_baz, \%opts_for_baz,
+   foo   => $type_for_foo, \%opts_for_foo,
+   bar   => $type_for_bar, \%opts_for_bar,
+   baz   => $type_for_baz, \%opts_for_baz,
    ...,
-   slurpy($slurpy_type),
+   extra => slurpy($slurpy_type),
  );
 
 The C<< $check >> coderef will return a hashref.
@@ -1195,10 +1377,50 @@ B<< Optional[Any] >> and B<< Any >>.
  state $check = compile_named(foo => 1, bar => 0, baz => 0);
  my $args = $check->(@_);  # $args->{bar} and $args->{baz} are optional
 
-Slurpy parameters are supported as you'd expect.
+Slurpy parameters are slurped into a nested hashref.
+
+  my $check = compile(
+    foo    => Str,
+    bar    => Optional[Str],
+    extra  => slurpy HashRef[Str],
+  );
+  my $args = $check->(foo => "aaa", quux => "bbb");
+  
+  print $args->{foo}, "\n";             # aaa
+  print $args->{extra}{quux}, "\n";     # bbb
 
 B<< slurpy Any >> is treated as B<< slurpy HashRef >>.
 
+The C<head> and C<tail> options are supported. This allows for a
+mixture of positional and named arguments, as long as the positional
+arguments are non-optional and at the head and tail of C<< @_ >>.
+
+  my $check = compile(
+    { head => [ Int, Int ], tail => [ CodeRef ] },
+    foo => Str,
+    bar => Str,
+    baz => Str,
+  );
+  
+  my ($int1, $int2, $args, $coderef)
+    = $check->( 666, 999, foo=>'x', bar=>'y', baz=>'z', sub {...} );
+  
+  say $args->{bar};  # 'y'
+
+This can be combined with C<named_to_list>:
+
+  my $check = compile(
+    { head => [ Int, Int ], tail => [ CodeRef ], named_to_list => 1 },
+    foo => Str,
+    bar => Str,
+    baz => Str,
+  );
+  
+  my ($int1, $int2, $foo, $bar, $baz, $coderef)
+    = $check->( 666, 999, foo=>'x', bar=>'y', baz=>'z', sub {...} );
+  
+  say $bar;  # 'y'
+
 =head3 C<< validate_named(\@_, @spec) >>
 
 Like C<compile> has C<validate>, C<compile_named> has C<validate_named>.
