@@ -11,7 +11,7 @@ BEGIN {
 
 BEGIN {
 	$Type::Tiny::AUTHORITY   = 'cpan:TOBYINK';
-	$Type::Tiny::VERSION     = '1.008005';
+	$Type::Tiny::VERSION     = '1.010003';
 	$Type::Tiny::XS_VERSION  = '0.016';
 }
 
@@ -211,6 +211,10 @@ sub new
 	my $class  = shift;
 	my %params = (@_==1) ? %{$_[0]} : @_;
 	
+	for (qw/ name display_name library /) {
+		$params{$_} = $params{$_}.'' if defined $params{$_};
+	}
+	
 	if (exists $params{parent}) {
 		$params{parent} = ref($params{parent}) =~ /^Type::Tiny\b/
 			? $params{parent}
@@ -1296,11 +1300,11 @@ sub can
 			my $method = $self->_lookup_my_method($1);
 			return $method if $method;
 		}
-	}
-
-	if ($self->{is_object} && $object_methods{$_[0]}) {
-		require Type::Tiny::ConstrainedObject;
-		return Type::Tiny::ConstrainedObject->can($_[0]);
+		if ($self->{is_object} && $object_methods{$_[0]})
+		{
+			require Type::Tiny::ConstrainedObject;
+			return Type::Tiny::ConstrainedObject->can($_[0]);
+		}
 	}
 	
 	return;
@@ -1324,13 +1328,13 @@ sub AUTOLOAD
 			my $method = $self->_lookup_my_method($1);
 			return &$method($self, @_) if $method;
 		}
-	}
-	
-	if ($self->{is_object} && $object_methods{$m}) {
-		require Type::Tiny::ConstrainedObject;
-		unshift @_, $self;
-		no strict 'refs';
-		goto \&{"Type::Tiny::ConstrainedObject::$m"};
+		if ($self->{is_object} && $object_methods{$m})
+		{
+			require Type::Tiny::ConstrainedObject;
+			unshift @_, $self;
+			no strict 'refs';
+			goto \&{"Type::Tiny::ConstrainedObject::$m"};
+		}
 	}
 	
 	_croak q[Can't locate object method "%s" via package "%s"], $m, ref($self)||$self;
@@ -1597,7 +1601,7 @@ a coercion.)
 
 =item C<< my_methods >>
 
-Experimenal hashref of additional methods that can be called on the type
+Experimental hashref of additional methods that can be called on the type
 constraint object.
 
 =back
