@@ -12,7 +12,7 @@ BEGIN {
 
 BEGIN {
 	$Types::Standard::AUTHORITY = 'cpan:TOBYINK';
-	$Types::Standard::VERSION   = '1.008005';
+	$Types::Standard::VERSION   = '1.010003';
 }
 
 $Types::Standard::VERSION =~ tr/_//d;
@@ -29,12 +29,13 @@ my $is_class_loaded;
 
 BEGIN {
 	$is_class_loaded = q{sub {
+		no strict 'refs';
 		return !!0 if ref $_[0];
 		return !!0 if not $_[0];
 		return !!0 if ref(do { my $tmpstr = $_[0]; \$tmpstr }) ne 'SCALAR';
-		my $stash = do { no strict 'refs'; \%{"$_[0]\::"} };
-		return !!1 if exists $stash->{'ISA'};
-		return !!1 if exists $stash->{'VERSION'};
+		my $stash = \%{"$_[0]\::"};
+		return !!1 if exists($stash->{'ISA'}) && *{$stash->{'ISA'}}{ARRAY} && @{$_[0].'::ISA'};
+		return !!1 if exists($stash->{'VERSION'});
 		foreach my $globref (values %$stash) {
 			return !!1
 				if ref \$globref eq 'GLOB'
@@ -235,8 +236,12 @@ my $_str = $meta->$add_core_type({
 my $_laxnum = $meta->add_type({
 	name       => "LaxNum",
 	parent     => $_str,
-	constraint => sub { looks_like_number $_ },
-	inlined    => sub { "defined($_[1]) && !ref($_[1]) && Scalar::Util::looks_like_number($_[1])" },
+	constraint => sub { looks_like_number($_) and ref(\$_) ne 'GLOB' },
+	inlined    => sub {
+		'Scalar::Util'->VERSION('1.18') # RT 132426
+			? "defined($_[1]) && !ref($_[1]) && Scalar::Util::looks_like_number($_[1])"
+			: "defined($_[1]) && !ref($_[1]) && Scalar::Util::looks_like_number($_[1]) && ref(\\($_[1])) ne 'GLOB'"
+	},
 });
 
 my $_strictnum = $meta->add_type({
@@ -309,8 +314,8 @@ my $_ref = $meta->$add_core_type({
 		return $meta->get_type('Ref') unless @_;
 		
 		my $reftype = shift;
-		Types::TypeTiny::StringLike->check($reftype)
-			or _croak("Parameter to Ref[`a] expected to be string; got $reftype");
+		$reftype =~ /^(SCALAR|ARRAY|HASH|CODE|REF|GLOB|LVALUE|FORMAT|IO|VSTRING|REGEXP|Regexp)$/i
+			or _croak("Parameter to Ref[`a] expected to be a Perl ref type; got $reftype");
 		
 		$reftype = "$reftype";
 		return sub {
@@ -994,6 +999,9 @@ B<< Int >>
 An integer; that is a string of digits 0 to 9, optionally prefixed with a
 hyphen-minus character.
 
+Expect inconsistent results for dualvars, and numbers too high (or negative
+numbers too low) for Perl to safely represent as an integer.
+
 =item *
 
 B<< ClassName >>
@@ -1156,7 +1164,7 @@ This module also exports a C<slurpy> function, which can be used as
 follows.
 
 It can cause additional trailing values in a B<Tuple> to be slurped
-into a structure and validated. For example, slurping into an arrayfef:
+into a structure and validated. For example, slurping into an arrayref:
 
    my $type = Tuple[Str, slurpy ArrayRef[Int]];
    
